%cavendish
% Analysis of the Cavendish experiment to measure the Universal
% Gravitational Constant G. Performed in Winchester College
% Laboratory P4 by Andy French and Tony Ayres, 19th December 2016.
%
% LAST UPDATED by Andy French 20th December 2016
%
% Note 2014 CODATA value for G is: 
% G = 6.67408(31) x 10^-11 m^3 kg^-1 s^-2
%
% https://en.wikipedia.org/wiki/Gravitational_constant
%
% From calculations:
%
% On tape measure scale on wall (mm)
%
% Position A eq: 3785
% Position B eq: 4070*     
% Wall perpendicular point: 5078
%
% *From curve fit. We actually 'measured' 4032).
%
% Perpendicular distance to wall: 6820
%
% Separations between positions A and B: 285
% 
% Oscillation period: 616s
%
% This yields G = 8.19 x 10^-11 m^3 kg^-1 s^-2  (23% out)

function cavendish

%CODATA G
Gtrue = 6.67408e-11;

%

%'Final' equilibrium point on wall scale /mm corresponding to Position B. (The
%Position A equivalent shall be inferred from the data).
xB_eq_measured = 4032;

%Caveat - this measurement was taken about an hour after the end of the
%experiment. However, it might not actually be the final position!
%Hence an inferred final position shall actually be computed as well, in a
%similar manner to the Position A scenario.

%Laboratory measurements made via a laser range finder. A small correction
%(w/2) is made to account for the laser spot from the range finder reflecting off
%the white ridge above the mirror. Attempts to reflect off the mirror
%typically yielded the distance to the wall behind the equipment!

%Width of pendulum casing /m (assume pendulum is exactly in the centre)
w = 29e-3;

%Perpendicular distance of mirror to wall /m
L0 = 6810.2e-3 + w/2;

%Distance of position B equilibrium position from perpendicular position /mm
dL0B = 1045;

%Perpenduicular position on scale /mm
x_perp = xB_eq_measured + dL0B;

%Distance of position B equilibrium position from mirror /m
LB = 6850e-3 + w/2;

%

%Parameters from LD Didactic datasheet

%Small mass /kg
m1 = 0.015;

%Large mass /kg
m2 = 1496.6e-3;

%Radius of pendulum /m
d = 50e-3;

%Distance between mass centres, assuming perfect central alignment of
%the torsional pendulum /m
b = 46.5e-3;

%Correction factor to take into account anti-torque moment of the second
%lead sphere
dd = sqrt( (2*d)^2 + b^2 );
K = 1/( 1- (b/dd)^3 );

%

%Number of (cubic spline) interpolation steps
N =1000;

%Max timescale for graph (and interpolated traces) /min
tmax = 100;

%Fontsize for graphs
fsize = 16;

%Read in Excel data. x is the displacement of the projected laser spot on a
%horizontal scale mounted on the laboratory wall. Distances are in mm.

[num,txt,raw] = xlsread('Cavendish Experiment.xls');

%At large mass position A

%Time in minutes since start of experiment
tA = (0:57)*0.5;  

%Position of laser spot on wall /mm
xA = [];
for n=7:64
    xA = [xA,raw{n,2}];
end

%Remove missing values
missing = isnan(xA);
xA(missing) = [];
tA(missing) = [];

%At large mass position B

%Time in seconds since start of experiment
tB = 29.5 + (0:64)*0.5;

%Position of laser spot on wall /mm
xB = [];
for n=66:130
    xB = [xB,raw{n,2}];
end

%Remove missing values
missing = isnan(xB);
xB(missing) = [];
tB(missing) = [];

%Plot laser position on wall /mm vs time /mins
fig = figure('name','Cavendish','color',[ 1 1 1 ] );
plot( tA, xA, 'r+', tB, xB,'b+' );
legend({'Position A','Position B'},'fontsize',fsize );
xlabel('time /min', 'fontsize',fsize)
ylabel('Laser spot position on wall /mm','fontsize',fsize)
grid on;
set(gca,'fontsize',fsize)
hold on;
xlim([0,tmax])

%

%Interpolate x vs t for both positions
tAi = linspace( tA(1),tA(end),N );
xAi = interp1( tA, xA, tAi, 'spline');
tBi = linspace( tB(1),tB(end),N );
xBi = interp1( tB, xB, tBi, 'spline');

%Overlay interpolated traces on graph
% plot( tAi, xAi, 'r-', tBi, xBi,'b-' );

%Find peaks
[tAip,xAip] = peakfinder(tAi,xAi,50);
[tBip,xBip] = peakfinder(tBi,xBi,50);

%Plot peaks
for n=1:numel(tAip)
    plot( tAip(n),xAip(n),'m*' );
end
for n=1:numel(tBip)
    plot( tBip(n),xBip(n),'g*' );
end

%Find troughs
[tAit,xAit] = troughfinder(tAi,xAi,50);
[tBit,xBit] = troughfinder(tBi,xBi,50);

%Plot troughs
for n=1:numel(tAit)
    plot( tAit(n),xAit(n),'m*' );
end
for n=1:numel(tBit)
    plot( tBit(n),xBit(n),'g*' );
end

%Determine oscillation period /min from peak separation
dtA_peak = diff(tAip);
dtB_peak = diff(tBip);

%Compute period /min from average of these differences
TA = mean(dtA_peak);
TB = mean(dtB_peak);

%Determine exponential decay constant for sinuoids for positions A and B
%Use first two peaks.

%Position A - we will infer the equilibrium position from this data
x0 = xAip(1);
x1 = xAit(2);
x2 = xAip(2);
x3 = xAit(3);
gammaA = (1/TA)*log( (x0-x1)/(x2-x3));
AA = (x0-x1)/(1+ exp(-gammaA*TA/2) );
xA_eq = x0 - AA;

%Position B

%Attempt 1 - This appears to yield an erroneous equilibrium point!
%Use measured equilibrium position instead of calculated version. Perform a
%vertical Least-Sqaures fit to the (peak or trough) absolute values of the laser position,
%with equilibrium firstly subtracted
t = [0,TB/2,TB,1.5*TB,2*TB];
y = [xBip(1),xBit(1),xBip(2),xBit(2),xBip(3)];
y = y - xB_eq_measured;
y = log( abs(y) );
graph = 1;
[m,c,p] = LSfit_vert(t,y,graph);
gammaB = -m;

%Position B - we will infer the equilibrium position from this data.
x0 = xBip(1);
x1 = xBit(1);
x2 = xBip(2);
x3 = xBit(2);
gammaB = (1/TB)*log( (x0-x1)/(x2-x3));
AB = (x0-x1)/(1+ exp(-gammaB*TB/2) );
xB_eq = x0 - AB;

%Now check again using Least Squares ...
t = [0,TB/2,TB,1.5*TB,2*TB];
y = [xBip(1),xBit(1),xBip(2),xBit(2),xBip(3)];
y = y - xB_eq;
y = log( abs(y) );
graph = 1;
[m,c,p] = LSfit_vert(t,y,graph);
gammaB = -m;

%Actually .... although there appears a misalignment for the first few
%oscillations, it would be better overall to use the actual measured
%equilibrium position. Since it is more for cosmetic purpose, allow the
%gammaB computed from the curve fit, but used xB_eq_measured instead
%of xB_eq

%

%Determine difference between A and B equilibrium positions /m
delta_x = (xB_eq_measured - xA_eq)*1e-3;

%Determine wall spacing /m between perpendicular position to wall and
%position A equilibrium
L1 = (x_perp - xA_eq)*1e-3;

%Determine times /mins corresponding to first peak of cosinuoids
t0_A = tAip(1);
t0_B = tBip(1);

%Define sinuoids for positions A and B
ttA = linspace(0,tmax,N);
xxA = AA*exp( -gammaA*(ttA-t0_A) ).*cos( 2*pi*(ttA-t0_A)/TA ) + xA_eq;
ttB = linspace(tBi(1),tmax,N);
xxB = AB*exp( -gammaB*(ttB-t0_B) ).*cos( 2*pi*(ttB-t0_B)/TB ) + xB_eq;

%Overlay interpolated traces on graph
figure(fig);
plot( ttA, xxA, 'r-', ttB, xxB,'b-' );

%Overlay equilibrium positions
plot( [0,tmax],[xA_eq,xA_eq],'m--' );
plot( [0,tmax],[xB_eq,xB_eq],'g--' );

%Compute G from data
T = mean([TA,TB])*60;

%Using approximation in LD Didactic note P1.1.3.1
G = (pi^2) * (b^2) * d * (delta_x)*L0 * (1/(L0^2 + L1^2)) * K * (1/m2) * (T^-2);

%Calculating angles directly
thetaA = atan( L1/L0 );

%Using inferred equilibrium position
thetaB = atan( (x_perp - xB_eq_measured)*1e-3/L0 );

%Using measured equilibrium position
thetaB = atan((5078-4032)*1e-3/L0);

%G calculation
G = (pi^2) * (b^2) * d * ( thetaA - thetaB ) * K * (1/m2) * (T^-2);

%Determine % error from true G
error_percent = 100*abs(G-Gtrue)/Gtrue;

%Graph title
title({'Cavendish experiment laser spot positions',...
    ['\Deltax = ',num2str(delta_x*1000,3),' mm, T = ',num2str(T,3),'s, G = ',...
    num2str(G,3),' m^3 kg^{-1} s^{-2}'],...
    ['Error from actual G = ',num2str(error_percent,2),'%']},'fontsize',fsize)

%Save graphs
print(gcf,'cavendish.png','-dpng','-r300');

%%

%Peak finder function.
function [xp,yp] = peakfinder(x,y,n)

%Step through y vs x and look for y values which are above the previous n and
%subsequent n positions.
xp = [];
yp = [];

%Check at least 2*n+1 data points
if numel(x)<(2*n+1)
    return
end
for i=n+1:numel(x)-n
    t=[];
    for j=1:n
        t = [t,y(i)>y(i-j)];
    end
    for j=1:n
        t = [t,y(i)>y(i+j)];
    end
    t = prod(t);    
    if t==1
        %Point is a local peak! Add to list
        xp = [xp,x(i)];
        yp = [yp,y(i)];
    end
end

%%

%Trough finder function.
function [xp,yp] = troughfinder(x,y,n)

%Step through y vs x and look for y values which are below the previous n and
%subsequent n positions.
xp = [];
yp = [];

%Check at least 2*n+1 data points
if numel(x)<(2*n+1)
    return
end
for i=n+1:numel(x)-n
    t=[];
    for j=1:n
        t = [t,y(i)<y(i-j)];
    end
    for j=1:n
        t = [t,y(i)<y(i+j)];
    end
    t = prod(t);    
    if t==1
        %Point is a local trough! Add to list
        xp = [xp,x(i)];
        yp = [yp,y(i)];
    end
end

%%

%Line of best fit y = mx + c minimizing sum of squares of y deviations
function [m,c,p] = LSfit_vert(x,y,graph)
m = cv(x,y)/var(x);
c = mean(y) - m * mean(x);
p = cv(x,y)/sqrt( var(x)*var(y) );

%Plot graph if option set
if graph == 1
    figure('color',[1 1 1],'name','Line of best fit (vertical)')
    fsize = 18;
    xx = linspace(min(x),max(x),1000);
    yfit = m*xx+c;
    plot(xx,yfit,'k',x,y,'r+');
    xlabel('x','fontsize',fsize)
    ylabel('y','fontsize',fsize);
    title(['Vert. Line of best fit y = ',num2str(m,3),'x + ',...
        num2str(c,3),' p = ',num2str(p,3)],'fontsize',fsize)
    set(gca,'fontsize',fsize);
    grid on
end

%%

%Line of best fit y = mx + c minimizing sum of squares of x deviations
function [m,c,p] = LSfit_horz(x,y,graph)

%Work out line of best fit of the for x = m*y + c
M = cv(x,y)/var(y);
C = mean(x) - M * mean(y);

%Rearrange to form y = x/M -C/M = m*x + c
m = 1/M;
c = -C/M;

%Product-moment correlation coefficient
p = cv(x,y)/sqrt( var(x)*var(y) );

%Plot graph if option set
if graph == 1
    figure('color',[1 1 1],'name','Line of best fit (horizontal)')
    fsize = 18;
    xx = linspace(min(x),max(x),1000);
    yfit = m*xx+c;
    plot(xx,yfit,'k',x,y,'r+');
    xlabel('x','fontsize',fsize)
    ylabel('y','fontsize',fsize);
    title(['Horz. Line of best fit y = ',num2str(m,3),'x + ',...
        num2str(c,3),' p = ',num2str(p,3)],'fontsize',fsize)
    set(gca,'fontsize',fsize);
    grid on
end

%%

%Covariance of data x and y
function c = cv(x,y)
N = length(x);
c = sum(x.*y)/N - mean(x)*mean(y);


%End of code
